# Copyright(c) 2019, The HiLearn developer (Yuanhua Huang)
from .version import __version__
from .base import match, id_mapping
from .stats import permutation_test
from .cross_validation import CrossValidation
from .mixture_model.mix_linear_regression import MixLinearRegression

from .plot import *


